@@ -11,18 +11,45 @@ def make(data=None, **kwargs):
     return qr.make_image()
 
 
+def _check_version(version):
+    if version < 1 or version > 40:
+        raise ValueError(
+            "Invalid version (was %s, expected 1 to 40)" % version)
+
+
+def _check_box_size(size):
+    if int(size) <= 0:
+        raise ValueError(
+            "Invalid box size (was %s, expected larger than 0)" % size)
+
+
+def _check_mask_pattern(mask_pattern):
+    if mask_pattern is None:
+        return
+    if not isinstance(mask_pattern, int):
+        raise TypeError(
+            "Invalid mask pattern (was %s, expected int)" % type(mask_pattern))
+    if mask_pattern < 0 or mask_pattern > 7:
+        raise ValueError(
+            "Mask pattern should be in range(8) (got %s)" % mask_pattern)
+
+
 class QRCode:
 
     def __init__(self, version=None,
                  error_correction=constants.ERROR_CORRECT_M,
                  box_size=10, border=4,
-                 image_factory=None):
+                 image_factory=None,
+                 mask_pattern=None):
+        _check_box_size(box_size)
         self.version = version and int(version)
         self.error_correction = int(error_correction)
         self.box_size = int(box_size)
         # Spec says border should be at least four boxes wide, but allow for
         # any (e.g. for producing printable QR codes).
         self.border = int(border)
+        _check_mask_pattern(mask_pattern)
+        self.mask_pattern = mask_pattern
         self.image_factory = image_factory
         if image_factory is not None:
             assert issubclass(image_factory, BaseImage)
@@ -49,7 +76,8 @@ class QRCode:
             self.data_list.append(data)
         else:
             if optimize:
-                self.data_list.extend(util.optimal_data_chunks(data))
+                self.data_list.extend(
+                    util.optimal_data_chunks(data, minimum=optimize))
             else:
                 self.data_list.append(util.QRData(data))
         self.data_cache = None
@@ -61,11 +89,15 @@ class QRCode:
         :param fit: If ``True`` (or if a size has not been provided), find the
             best fit for the data to avoid data overflow errors.
         """
-        if fit or not self.version:
+        if fit or (self.version is None):
             self.best_fit(start=self.version)
-        self.makeImpl(False, self.best_mask_pattern())
+        if self.mask_pattern is None:
+            self.makeImpl(False, self.best_mask_pattern())
+        else:
+            self.makeImpl(False, self.mask_pattern)
 
     def makeImpl(self, test, mask_pattern):
+        _check_version(self.version)
         self.modules_count = self.version * 4 + 17
         self.modules = [None] * self.modules_count
 
@@ -79,7 +111,7 @@ class QRCode:
         self.setup_position_probe_pattern(0, 0)
         self.setup_position_probe_pattern(self.modules_count - 7, 0)
         self.setup_position_probe_pattern(0, self.modules_count - 7)
-        self.sutup_position_adjust_pattern()
+        self.setup_position_adjust_pattern()
         self.setup_timing_pattern()
         self.setup_type_info(test, mask_pattern)
 
@@ -115,9 +147,7 @@ class QRCode:
         """
         if start is None:
             start = 1
-        elif start < 1 or start > 40:
-            raise ValueError("Invalid version (was %s, expected 1 to 40)" %
-                version)
+        _check_version(start)
 
         # Corresponds to the code in util.create_data, except we don't yet know
         # version, so optimistically assume start and check later
@@ -196,7 +226,14 @@ class QRCode:
         """
         if out is None:
             import sys
-            out = sys.stdout
+            if sys.version_info < (2, 7):
+                # On Python versions 2.6 and earlier, stdout tries to encode
+                # strings using ASCII rather than stdout.encoding, so use this
+                # workaround.
+                import codecs
+                out = codecs.getwriter(sys.stdout.encoding)(sys.stdout)
+            else:
+                out = sys.stdout
 
         if tty and not out.isatty():
             raise OSError("Not a tty")
@@ -205,8 +242,8 @@ class QRCode:
             self.make()
 
         modcount = self.modules_count
-        codes = [
-            chr(code).decode('cp437') for code in (255, 223, 220, 219)]
+        codes = [six.int2byte(code).decode('cp437')
+                 for code in (255, 223, 220, 219)]
         if tty:
             invert = True
         if invert:
@@ -239,6 +276,7 @@ class QRCode:
 
         If the data has not been compiled yet, make it first.
         """
+        _check_box_size(self.box_size)
         if self.data_cache is None:
             self.make()
 
@@ -270,7 +308,7 @@ class QRCode:
                 continue
             self.modules[6][c] = (c % 2 == 0)
 
-    def sutup_position_adjust_pattern(self):
+    def setup_position_adjust_pattern(self):
         pos = util.pattern_position(self.version)
 
         for i in range(len(pos)):
